package liste;

/**
 * Implementazione di liste tramite puntatori (references)
 * 
 * @author Stefano Guerrini
 * @version Marzo 2009
 */
public class Nodo
{
    // instance variables
    private EtichettaNodo label;
    private Nodo next;

    //
    //  Costruttori
    //

    /**
     * Costruttore di oggetti della classe Nodo
     * 
     * Crea una lista di un solo elemento
     * 
     * Utilizza il costruttore senza parametri di EtichettaNodo
     * per inserire un elemento nel nodo
     */
    public Nodo()
    {
        label = new EtichettaNodo();
        next = null;
    }

    /**
     * Costruttore di oggetti della classe Nodo
     * 
     * Crea una lista di un solo elemento
     * 
     * @param l etichetta del nuovo Nodo
     */
    public Nodo(EtichettaNodo l)
    {
        label = l;
        next = null;
    }
    
    /**
     * Costruttore di oggetti della classe Nodo
     * 
     * Aggiunge un nuovo nodo in testa a una lista
     * 
     * @param l     etichetta del nuovo Nodo
     * @param nd    lista a cui aggiungere il nuovo nodo
     * */
    public Nodo(EtichettaNodo l, Nodo nd)
    {
        // initialise instance variables
        label = l;
        next = nd;
    }

    //
    //  getters
    //

    /**
     * Metodo getter: restituisce la coda della lista di cui
     * il nodo  il primo elemento
     */
    public Nodo next()
    {
        return next;
    }

    /**
     * Metodo getter: restituisce l'etichetta del nodo
     */
    public EtichettaNodo label()
    {
        return this.label;
    }

    //
    //  setters
    //

    /**
     * Metodo setter: cambia il valore della coda della lista di cui
     * il nodo  il primo elemento
     * 
     * @param nd    nuova coda della lista
     */
    public void setNext(Nodo nd)
    {
        next = nd;
    }

    /**
     * Metodo setter: cambia il valore dell'etichetta del nodo
     * 
     * @param v     nuova etichetta
     */
    public void setLabel(EtichettaNodo v)
    {
        this.label = v;
    }

    //
    // metodi statici
    //
    
    /**
     * lunghezza di una lista
     * 
     * @param nd    lista di cui calcolare la lunghezza
     * @return      la lunghezza della lista
     */
    static public int lunghezza(Nodo nd)
    {
        int n = 0;
        for (; nd != null; nd = nd.next(), n++);
        return n;
    }

    /**
     * lunghezza di una lista
     * (versione ricorsiva)
     * 
     * @param nd    lista di cui calcolare la lunghezza
     * @return      la lunghezza della lista
     */
    static public int lunghRic(Nodo nd)
    {
        if (nd == null)
            return 0;
        else
            return 1 + lunghRic(nd.next());
    }
    
    /**
     * appende una lista ad un'altra lista
     * 
     * @param nd1   la lista a cui appendere
     * @param nd2   la lista da appendere
     * @return      la lista risultato. Se nd1 non e' vuota, e' pari a nd1
     *              (nd2 e' stata attaccata all'ultimo elemento di nd1),
     *              altrimenti e' pari a nd2
     */
    public static Nodo append(Nodo nd1, Nodo nd2)
    {
        if (nd1 == null)
            return nd2;
        else
        {
            nd1.append(nd2); // usa append definita come instance method
            return nd1;
        }
    }

    /**
     * somma i valori interi degli elementi di una lista
     * il valore intero di una etichetta e' quello restituito da getIntVal
     * 
     * @return      la somma
     */
    public static int somma(Nodo nd)
    {
        int acc = 0;
        
        for ( ; nd != null ; nd = nd.next() )
            acc += nd.label().getIntVal();

        return acc;
    }

    //
    // metodi delle istanze
    //
    
    /**
     * lunghezza della lista di cui il nodo  il primo elemento
     * 
     * @return      la lunghezza della lista
     */
    public int lunghezza()
    {
        return (Nodo.lunghezza(this)); // usa la version static
    }

    /**
     * copia una lista (le etichette sono clonate)
     * 
     * @return      la lista copia
     */
    public Nodo copia()
    {
        return new Nodo(label.clona(), next == null ? null : next.copia());
    }

    /**
     * rovescia una lista (senza clonare le etichette)
     * 
     * @return      la lista rovesciata
     */
    public Nodo reverse()
    {
        Nodo nuova = null;
        Nodo nd = this;
        
        while (nd != null) 
        {
            Nodo aux = nd.next();
            nd.setNext(nuova);
            nuova = nd;
            nd = aux;
        } 

        return nuova;
    }

    /**
     * appende una lista alla lista di cui il nodo  il primo elemento
     * 
     * @param nd    la lista da appendere
     */
    public void append(Nodo nd)
    {
        if (next == null)
            next = nd;
        else
            next.append(nd);
    }
    
    /**
     * somma i valori interi degli elementi di una lista
     * il valore intero di una etichetta e' quello restituito da getIntVal
     * 
     * @return      la somma
     */
    public int somma()
    {
        if (next == null)
            return label.getIntVal();
        else
            return label.getIntVal()+next.somma();
    }          
}
